@@ -7,7 +7,7 @@
 #  define VTKIOPARALLELXDMF3_NO_EXPORT
 #else
 #  ifndef VTKIOPARALLELXDMF3_EXPORT
-#    ifdef vtkIOParallelXdmf3_EXPORTS
+#    ifdef IOParallelXdmf3_EXPORTS
         /* We are building this library */
 #      define VTKIOPARALLELXDMF3_EXPORT __attribute__((visibility("default")))
 #    else
@@ -23,26 +23,32 @@
 
 #ifndef VTKIOPARALLELXDMF3_DEPRECATED
 #  define VTKIOPARALLELXDMF3_DEPRECATED __attribute__ ((__deprecated__))
-#  define VTKIOPARALLELXDMF3_DEPRECATED_EXPORT VTKIOPARALLELXDMF3_EXPORT __attribute__ ((__deprecated__))
-#  define VTKIOPARALLELXDMF3_DEPRECATED_NO_EXPORT VTKIOPARALLELXDMF3_NO_EXPORT __attribute__ ((__deprecated__))
 #endif
 
-#define DEFINE_NO_DEPRECATED 0
-#if DEFINE_NO_DEPRECATED
-# define VTKIOPARALLELXDMF3_NO_DEPRECATED
+#ifndef VTKIOPARALLELXDMF3_DEPRECATED_EXPORT
+#  define VTKIOPARALLELXDMF3_DEPRECATED_EXPORT VTKIOPARALLELXDMF3_EXPORT VTKIOPARALLELXDMF3_DEPRECATED
 #endif
 
-/* AutoInit dependencies.  */
-#include "vtkIOXdmf3Module.h"
+#ifndef VTKIOPARALLELXDMF3_DEPRECATED_NO_EXPORT
+#  define VTKIOPARALLELXDMF3_DEPRECATED_NO_EXPORT VTKIOPARALLELXDMF3_NO_EXPORT VTKIOPARALLELXDMF3_DEPRECATED
+#endif
+
+#if 0 /* DEFINE_NO_DEPRECATED */
+#  ifndef VTKIOPARALLELXDMF3_NO_DEPRECATED
+#    define VTKIOPARALLELXDMF3_NO_DEPRECATED
+#  endif
+#endif
+/* AutoInit dependencies. */
 #include "vtkIOXdmf3Module.h"
 
-/* AutoInit implementations.  */
-#if defined(vtkIOParallelXdmf3_INCLUDE)
-# include vtkIOParallelXdmf3_INCLUDE
+
+/* AutoInit implementations. */
+#ifdef vtkIOParallelXdmf3_AUTOINIT_INCLUDE
+#include vtkIOParallelXdmf3_AUTOINIT_INCLUDE
 #endif
-#if defined(vtkIOParallelXdmf3_AUTOINIT)
-# include "vtkAutoInit.h"
-VTK_AUTOINIT(vtkIOParallelXdmf3)
+#ifdef vtkIOParallelXdmf3_AUTOINIT
+#include "vtkAutoInit.h"
+VTK_MODULE_AUTOINIT(vtkIOParallelXdmf3)
 #endif
 
-#endif
+#endif /* VTKIOPARALLELXDMF3_EXPORT_H */
