@@ -27,16 +27,17 @@
  * PLOT3D files consist of a grid file (also known as XYZ file), an
  * optional solution file (also known as a Q file), and an optional function
  * file that contains user created data (currently unsupported). The Q file
- * contains solution  information as follows: the four parameters free stream
+ * contains solution information as follows: the four parameters free stream
  * mach number (Fsmach), angle of attack (Alpha), Reynolds number (Re), and
  * total integration time (Time). This information is stored in an array
  * called Properties in the FieldData of each output (tuple 0: fsmach, tuple 1:
  * alpha, tuple 2: re, tuple 3: time). In addition, the solution file contains
  * the flow density (scalar), flow momentum (vector), and flow energy (scalar).
  *
- * Note that this reader does not support time series data which is usually
- * stored as a series of Q and optionally XYZ files. If you want to read such
- * a file series, use vtkPlot3DMetaReader.
+ * This reader supports a limited form of time series data which are stored
+ * as a series of Q files. Using the AddFileName() method provided by the
+ * superclass, one can define a file series. For other cases, for example where
+ * the XYZ or function files vary over time, use vtkPlot3DMetaReader.
  *
  * The reader can generate additional scalars and vectors (or "functions")
  * from this information. To use vtkMultiBlockPLOT3DReader, you must specify the
@@ -81,14 +82,14 @@
  *
  * @sa
  * vtkMultiBlockDataSet vtkStructuredGrid vtkPlot3DMetaReader
-*/
+ */
 
 #ifndef vtkMultiBlockPLOT3DReader_h
 #define vtkMultiBlockPLOT3DReader_h
 
-#include <vector>  // For holding function-names
 #include "vtkIOParallelModule.h" // For export macro
-#include "vtkMultiBlockDataSetAlgorithm.h"
+#include "vtkParallelReader.h"
+#include <vector> // For holding function-names
 
 class vtkDataArray;
 class vtkDataSetAttributes;
@@ -98,78 +99,95 @@ class vtkMultiProcessController;
 class vtkStructuredGrid;
 class vtkUnsignedCharArray;
 struct vtkMultiBlockPLOT3DReaderInternals;
+class vtkMultiBlockDataSet;
 
 namespace Functors
 {
-  class ComputeFunctor;
-  class ComputeTemperatureFunctor;
-  class ComputePressureFunctor;
-  class ComputePressureCoefficientFunctor;
-  class ComputeMachNumberFunctor;
-  class ComputeSoundSpeedFunctor;
-  class ComputeEnthalpyFunctor;
-  class ComputeKinecticEnergyFunctor;
-  class ComputeVelocityMagnitudeFunctor;
-  class ComputeEntropyFunctor;
-  class ComputeSwirlFunctor;
-  class ComputeVelocityFunctor;
-  class ComputeVorticityMagnitudeFunctor;
-  class ComputePressureGradientFunctor;
-  class ComputeVorticityFunctor;
-  class ComputeStrainRateFunctor;
+class ComputeFunctor;
+class ComputeTemperatureFunctor;
+class ComputePressureFunctor;
+class ComputePressureCoefficientFunctor;
+class ComputeMachNumberFunctor;
+class ComputeSoundSpeedFunctor;
+class ComputeEnthalpyFunctor;
+class ComputeKinecticEnergyFunctor;
+class ComputeVelocityMagnitudeFunctor;
+class ComputeEntropyFunctor;
+class ComputeSwirlFunctor;
+class ComputeVelocityFunctor;
+class ComputeVorticityMagnitudeFunctor;
+class ComputePressureGradientFunctor;
+class ComputeVorticityFunctor;
+class ComputeStrainRateFunctor;
 }
 
-
-class VTKIOPARALLEL_EXPORT vtkMultiBlockPLOT3DReader : public vtkMultiBlockDataSetAlgorithm
+class VTKIOPARALLEL_EXPORT vtkMultiBlockPLOT3DReader : public vtkParallelReader
 {
-friend class Functors::ComputeFunctor;
-friend class Functors::ComputeTemperatureFunctor;
-friend class Functors::ComputePressureFunctor;
-friend class Functors::ComputePressureCoefficientFunctor;
-friend class Functors::ComputeMachNumberFunctor;
-friend class Functors::ComputeSoundSpeedFunctor;
-friend class Functors::ComputeEnthalpyFunctor;
-friend class Functors::ComputeKinecticEnergyFunctor;
-friend class Functors::ComputeVelocityMagnitudeFunctor;
-friend class Functors::ComputeEntropyFunctor;
-friend class Functors::ComputeSwirlFunctor;
-friend class Functors::ComputeVelocityFunctor;
-friend class Functors::ComputeVorticityMagnitudeFunctor;
-friend class Functors::ComputePressureGradientFunctor;
-friend class Functors::ComputeVorticityFunctor;
-friend class Functors::ComputeStrainRateFunctor;
+  friend class Functors::ComputeFunctor;
+  friend class Functors::ComputeTemperatureFunctor;
+  friend class Functors::ComputePressureFunctor;
+  friend class Functors::ComputePressureCoefficientFunctor;
+  friend class Functors::ComputeMachNumberFunctor;
+  friend class Functors::ComputeSoundSpeedFunctor;
+  friend class Functors::ComputeEnthalpyFunctor;
+  friend class Functors::ComputeKinecticEnergyFunctor;
+  friend class Functors::ComputeVelocityMagnitudeFunctor;
+  friend class Functors::ComputeEntropyFunctor;
+  friend class Functors::ComputeSwirlFunctor;
+  friend class Functors::ComputeVelocityFunctor;
+  friend class Functors::ComputeVorticityMagnitudeFunctor;
+  friend class Functors::ComputePressureGradientFunctor;
+  friend class Functors::ComputeVorticityFunctor;
+  friend class Functors::ComputeStrainRateFunctor;
+
 public:
-  static vtkMultiBlockPLOT3DReader *New();
-  vtkTypeMacro(vtkMultiBlockPLOT3DReader,vtkMultiBlockDataSetAlgorithm);
+  static vtkMultiBlockPLOT3DReader* New();
+  vtkTypeMacro(vtkMultiBlockPLOT3DReader, vtkParallelReader);
   void PrintSelf(ostream& os, vtkIndent indent) override;
 
-  //@{
+  ///@{
+  /**
+   * Get the output data object for a port on this algorithm.
+   */
+  vtkMultiBlockDataSet* GetOutput();
+  vtkMultiBlockDataSet* GetOutput(int);
+  ///@}
+
+  ///@{
   /**
    * Set/Get the PLOT3D geometry filename.
    */
-  void SetFileName(const char* name) { this->SetXYZFileName(name); }
-  const char* GetFileName() { return this->GetXYZFileName(); }
-  virtual void SetXYZFileName( const char* );
-  vtkGetStringMacro(XYZFileName);
-  //@}
+  void SetFileName(VTK_FILEPATH const char* name) { this->SetXYZFileName(name); }
+  VTK_FILEPATH const char* GetFileName() { return this->GetXYZFileName(); }
+  VTK_FILEPATH const char* GetFileName(int i) { return this->vtkParallelReader::GetFileName(i); }
+  virtual void SetXYZFileName(VTK_FILEPATH const char*);
+  vtkGetFilePathMacro(XYZFileName);
+  ///@}
 
-  //@{
+  ///@{
   /**
-   * Set/Get the PLOT3D solution filename.
+   * Set/Get the PLOT3D solution filename. This adds a filename
+   * using the superclass' AddFileName() method. To read a series
+   * of q files, use the AddFileName() interface directly to add
+   * multiple q filenames in the appropriate order. If the files
+   * are of Overflow format, the reader will read the time values
+   * from the files. Otherwise, it will use an integer sequence.
+   * Use a meta reader to support time values for non-Overflow file
+   * sequences.
    */
-  vtkSetStringMacro(QFileName);
-  vtkGetStringMacro(QFileName);
-  //@}
+  void SetQFileName(VTK_FILEPATH const char* name);
+  VTK_FILEPATH const char* GetQFileName();
+  ///@}
 
-  //@{
+  ///@{
   /**
    * Set/Get the PLOT3D function filename.
    */
-  vtkSetStringMacro(FunctionFileName);
-  vtkGetStringMacro(FunctionFileName);
-  //@}
+  vtkSetFilePathMacro(FunctionFileName);
+  vtkGetFilePathMacro(FunctionFileName);
+  ///@}
 
-  //@{
+  ///@{
   /**
    * When this option is turned on, the reader will try to figure
    * out the values of various options such as byte order, byte
@@ -180,89 +198,89 @@ public:
    * that you turn on AutoDetectFormat and leave the other file format
    * related options untouched.
    */
-  vtkSetMacro(AutoDetectFormat, int);
-  vtkGetMacro(AutoDetectFormat, int);
-  vtkBooleanMacro(AutoDetectFormat, int);
-  //@}
+  vtkSetMacro(AutoDetectFormat, vtkTypeBool);
+  vtkGetMacro(AutoDetectFormat, vtkTypeBool);
+  vtkBooleanMacro(AutoDetectFormat, vtkTypeBool);
+  ///@}
 
-  //@{
+  ///@{
   /**
    * Is the file to be read written in binary format (as opposed
    * to ascii).
    */
-  vtkSetMacro(BinaryFile, int);
-  vtkGetMacro(BinaryFile, int);
-  vtkBooleanMacro(BinaryFile, int);
-  //@}
+  vtkSetMacro(BinaryFile, vtkTypeBool);
+  vtkGetMacro(BinaryFile, vtkTypeBool);
+  vtkBooleanMacro(BinaryFile, vtkTypeBool);
+  ///@}
 
-  //@{
+  ///@{
   /**
    * Does the file to be read contain information about number of
    * grids. In some PLOT3D files, the first value contains the number
    * of grids (even if there is only 1). If reading such a file,
    * set this to true.
    */
-  vtkSetMacro(MultiGrid, int);
-  vtkGetMacro(MultiGrid, int);
-  vtkBooleanMacro(MultiGrid, int);
-  //@}
+  vtkSetMacro(MultiGrid, vtkTypeBool);
+  vtkGetMacro(MultiGrid, vtkTypeBool);
+  vtkBooleanMacro(MultiGrid, vtkTypeBool);
+  ///@}
 
-  //@{
+  ///@{
   /**
    * Were the arrays written with leading and trailing byte counts ?
    * Usually, files written by a fortran program will contain these
    * byte counts whereas the ones written by C/C++ won't.
    */
-  vtkSetMacro(HasByteCount, int);
-  vtkGetMacro(HasByteCount, int);
-  vtkBooleanMacro(HasByteCount, int);
-  //@}
+  vtkSetMacro(HasByteCount, vtkTypeBool);
+  vtkGetMacro(HasByteCount, vtkTypeBool);
+  vtkBooleanMacro(HasByteCount, vtkTypeBool);
+  ///@}
 
-  //@{
+  ///@{
   /**
    * Is there iblanking (point visibility) information in the file.
    * If there is iblanking arrays, these will be read and assigned
    * to the PointVisibility array of the output.
    */
-  vtkSetMacro(IBlanking, int);
-  vtkGetMacro(IBlanking, int);
-  vtkBooleanMacro(IBlanking, int);
-  //@}
+  vtkSetMacro(IBlanking, vtkTypeBool);
+  vtkGetMacro(IBlanking, vtkTypeBool);
+  vtkBooleanMacro(IBlanking, vtkTypeBool);
+  ///@}
 
-  //@{
+  ///@{
   /**
    * If only two-dimensional data was written to the file,
    * turn this on.
    */
-  vtkSetMacro(TwoDimensionalGeometry, int);
-  vtkGetMacro(TwoDimensionalGeometry, int);
-  vtkBooleanMacro(TwoDimensionalGeometry, int);
-  //@}
+  vtkSetMacro(TwoDimensionalGeometry, vtkTypeBool);
+  vtkGetMacro(TwoDimensionalGeometry, vtkTypeBool);
+  vtkBooleanMacro(TwoDimensionalGeometry, vtkTypeBool);
+  ///@}
 
-  //@{
+  ///@{
   /**
    * Is this file in double precision or single precision.
    * This only matters for binary files.
    * Default is single.
    */
-  vtkSetMacro(DoublePrecision, int);
-  vtkGetMacro(DoublePrecision, int);
-  vtkBooleanMacro(DoublePrecision, int);
-  //@}
+  vtkSetMacro(DoublePrecision, vtkTypeBool);
+  vtkGetMacro(DoublePrecision, vtkTypeBool);
+  vtkBooleanMacro(DoublePrecision, vtkTypeBool);
+  ///@}
 
-  //@{
+  ///@{
   /**
    * Try to read a binary file even if the file length seems to be
    * inconsistent with the header information. Use this with caution,
    * if the file length is not the same as calculated from the header.
    * either the file is corrupt or the settings are wrong.
    */
-  vtkSetMacro(ForceRead, int);
-  vtkGetMacro(ForceRead, int);
-  vtkBooleanMacro(ForceRead, int);
-  //@}
+  vtkSetMacro(ForceRead, vtkTypeBool);
+  vtkGetMacro(ForceRead, vtkTypeBool);
+  vtkBooleanMacro(ForceRead, vtkTypeBool);
+  ///@}
 
-  //@{
+  ///@{
   /**
    * Set the byte order of the file (remember, more Unix workstations
    * write big endian whereas PCs write little endian). Default is
@@ -273,26 +291,26 @@ public:
   void SetByteOrderToLittleEndian();
   vtkSetMacro(ByteOrder, int);
   vtkGetMacro(ByteOrder, int);
-  const char *GetByteOrderAsString();
-  //@}
+  const char* GetByteOrderAsString();
+  ///@}
 
-  //@{
+  ///@{
   /**
    * Set/Get the gas constant. Default is 1.0.
    */
-  vtkSetMacro(R,double);
-  vtkGetMacro(R,double);
-  //@}
+  vtkSetMacro(R, double);
+  vtkGetMacro(R, double);
+  ///@}
 
-  //@{
+  ///@{
   /**
    * Set/Get the ratio of specific heats. Default is 1.4.
    */
-  vtkSetMacro(Gamma,double);
-  vtkGetMacro(Gamma,double);
-  //@}
+  vtkSetMacro(Gamma, double);
+  vtkGetMacro(Gamma, double);
+  ///@}
 
-  //@{
+  ///@{
   /**
    * When set to true (default), the reader will preserve intermediate computed
    * quantities that were not explicitly requested e.g. if `VelocityMagnitude` is
@@ -305,25 +323,25 @@ public:
   vtkGetMacro(PreserveIntermediateFunctions, bool);
   vtkBooleanMacro(PreserveIntermediateFunctions, bool);
 
-  //@{
+  ///@{
   /**
    * Specify the scalar function to extract. If ==(-1), then no scalar
    * function is extracted.
    */
   void SetScalarFunctionNumber(int num);
-  vtkGetMacro(ScalarFunctionNumber,int);
-  //@}
+  vtkGetMacro(ScalarFunctionNumber, int);
+  ///@}
 
-  //@{
+  ///@{
   /**
    * Specify the vector function to extract. If ==(-1), then no vector
    * function is extracted.
    */
   void SetVectorFunctionNumber(int num);
-  vtkGetMacro(VectorFunctionNumber,int);
-  //@}
+  vtkGetMacro(VectorFunctionNumber, int);
+  ///@}
 
-  //@{
+  ///@{
   /**
    * Specify additional functions to read. These are placed into the
    * point data as data arrays. Later on they can be used by labeling
@@ -332,64 +350,78 @@ public:
   void AddFunction(int functionNumber);
   void RemoveFunction(int);
   void RemoveAllFunctions();
-  //@}
+  ///@}
 
   /**
    * Return 1 if the reader can read the given file name. Only meaningful
    * for binary files.
    */
-  virtual int CanReadBinaryFile(const char* fname);
+  virtual int CanReadBinaryFile(VTK_FILEPATH const char* fname);
 
-  //@{
+  ///@{
   /**
    * Set/Get the communicator object (we'll use global World controller
    * if you don't set a different one).
    */
-  void SetController(vtkMultiProcessController *c);
+  void SetController(vtkMultiProcessController* c);
   vtkGetObjectMacro(Controller, vtkMultiProcessController);
-  //@}
+  ///@}
 
-  void AddFunctionName(const std::string &name) {FunctionNames.push_back(name);}
+  void AddFunctionName(const std::string& name) { FunctionNames.push_back(name); }
 
   enum
   {
-    FILE_BIG_ENDIAN=0,
-    FILE_LITTLE_ENDIAN=1
+    FILE_BIG_ENDIAN = 0,
+    FILE_LITTLE_ENDIAN = 1
   };
 
+  ///@{
+  /**
+   * These methods have to be overwritten from superclass
+   * because Plot3D actually uses the XYZ file to read these.
+   * This is not recognized by the superclass which returns
+   * an error when a filename (Q filename) is not set.
+   */
+  int ReadMetaData(vtkInformation* metadata) override;
+  int ReadMesh(int piece, int npieces, int nghosts, int timestep, vtkDataObject* output) override;
+  int ReadPoints(int piece, int npieces, int nghosts, int timestep, vtkDataObject* output) override;
+  int ReadArrays(int piece, int npieces, int nghosts, int timestep, vtkDataObject* output) override;
+  ///@}
+
 protected:
   vtkMultiBlockPLOT3DReader();
   ~vtkMultiBlockPLOT3DReader() override;
 
+  ///@{
+  /**
+   * Overridden from superclass to do actual reading.
+   */
+  double GetTimeValue(const std::string& fname) override;
+  int ReadMesh(
+    const std::string& fname, int piece, int npieces, int nghosts, vtkDataObject* output) override;
+  int ReadPoints(
+    const std::string& fname, int piece, int npieces, int nghosts, vtkDataObject* output) override;
+  int ReadArrays(
+    const std::string& fname, int piece, int npieces, int nghosts, vtkDataObject* output) override;
+  ///@}
+
   vtkDataArray* CreateFloatArray();
 
   int CheckFile(FILE*& fp, const char* fname);
   int CheckGeometryFile(FILE*& xyzFp);
-  int CheckSolutionFile(FILE*& qFp);
   int CheckFunctionFile(FILE*& fFp);
 
   int GetByteCountSize();
-  int SkipByteCount (FILE* fp);
-  int ReadIntBlock  (FILE* fp, int n, int*   block);
+  int SkipByteCount(FILE* fp);
+  int ReadIntBlock(FILE* fp, int n, int* block);
 
-  vtkIdType ReadValues(
-    FILE* fp,
-    int n,
-    vtkDataArray* scalar);
-  virtual int ReadIntScalar(
-    void* vfp,
-    int extent[6], int wextent[6],
-    vtkDataArray* scalar, vtkTypeUInt64 offset,
-    const vtkMultiBlockPLOT3DReaderRecord& currentRecord);
-  virtual int ReadScalar(
-    void* vfp,
-    int extent[6], int wextent[6],
-    vtkDataArray* scalar, vtkTypeUInt64 offset,
-    const vtkMultiBlockPLOT3DReaderRecord& currentRecord);
-  virtual int ReadVector(
-    void* vfp,
-    int extent[6], int wextent[6],
-    int numDims, vtkDataArray* vector, vtkTypeUInt64 offset,
+  vtkIdType ReadValues(FILE* fp, int n, vtkDataArray* scalar);
+  virtual int ReadIntScalar(void* vfp, int extent[6], int wextent[6], vtkDataArray* scalar,
+    vtkTypeUInt64 offset, const vtkMultiBlockPLOT3DReaderRecord& currentRecord);
+  virtual int ReadScalar(void* vfp, int extent[6], int wextent[6], vtkDataArray* scalar,
+    vtkTypeUInt64 offset, const vtkMultiBlockPLOT3DReaderRecord& currentRecord);
+  virtual int ReadVector(void* vfp, int extent[6], int wextent[6], int numDims,
+    vtkDataArray* vector, vtkTypeUInt64 offset,
     const vtkMultiBlockPLOT3DReaderRecord& currentRecord);
   virtual int OpenFileForDataRead(void*& fp, const char* fname);
   virtual void CloseFile(void* fp);
@@ -404,11 +436,10 @@ protected:
 
   int AutoDetectionCheck(FILE* fp);
 
-  void AssignAttribute(int fNumber, vtkStructuredGrid* output,
-                       int attributeType);
+  void AssignAttribute(int fNumber, vtkStructuredGrid* output, int attributeType);
   void MapFunction(int fNumber, vtkStructuredGrid* output);
 
-  //@{
+  ///@{
   /**
    * Each of these methods compute a derived quantity. On success, the array is
    * added to the output and a pointer to the same is returned.
@@ -428,7 +459,7 @@ protected:
   vtkDataArray* ComputeSoundSpeed(vtkStructuredGrid* output);
   vtkDataArray* ComputeVorticityMagnitude(vtkStructuredGrid* output);
   vtkDataArray* ComputeStrainRate(vtkStructuredGrid* output);
-  //@}
+  ///@}
 
   // Returns a vtkFloatArray or a vtkDoubleArray depending
   // on DoublePrecision setting
@@ -441,53 +472,45 @@ protected:
 
   double GetGamma(vtkIdType idx, vtkDataArray* gamma);
 
-  //plot3d FileNames
-  char *XYZFileName;
-  char *QFileName;
-  char *FunctionFileName;
+  int FillOutputPortInformation(int port, vtkInformation* info) override;
 
-  int BinaryFile;
-  int HasByteCount;
-  int TwoDimensionalGeometry;
-  int MultiGrid;
-  int ForceRead;
+  // plot3d FileNames
+  char* XYZFileName;
+  char* QFileName;
+  char* FunctionFileName;
+  vtkTypeBool BinaryFile;
+  vtkTypeBool HasByteCount;
+  vtkTypeBool TwoDimensionalGeometry;
+  vtkTypeBool MultiGrid;
+  vtkTypeBool ForceRead;
   int ByteOrder;
-  int IBlanking;
-  int DoublePrecision;
-  int AutoDetectFormat;
+  vtkTypeBool IBlanking;
+  vtkTypeBool DoublePrecision;
+  vtkTypeBool AutoDetectFormat;
 
   int ExecutedGhostLevels;
 
   size_t FileSize;
 
-  //parameters used in computing derived functions
+  // parameters used in computing derived functions
   double R;
   double Gamma;
   double GammaInf;
 
   bool PreserveIntermediateFunctions;
 
-  //named functions from meta data
+  // named functions from meta data
   std::vector<std::string> FunctionNames;
 
-  //functions to read that are not scalars or vectors
-  vtkIntArray *FunctionList;
+  // functions to read that are not scalars or vectors
+  vtkIntArray* FunctionList;
 
   int ScalarFunctionNumber;
   int VectorFunctionNumber;
 
-  int FillOutputPortInformation(int port, vtkInformation* info) override;
-
-  int RequestData(vtkInformation*,
-                          vtkInformationVector**,
-                          vtkInformationVector*) override;
-  int RequestInformation(vtkInformation*,
-                                 vtkInformationVector**,
-                                 vtkInformationVector*) override;
-
   vtkMultiBlockPLOT3DReaderInternals* Internal;
 
-  vtkMultiProcessController *Controller;
+  vtkMultiProcessController* Controller;
 
 private:
   vtkMultiBlockPLOT3DReader(const vtkMultiBlockPLOT3DReader&) = delete;
